#!/usr/bin/env python

# Copyright (C) 2017-2020 Alex Manuskin, Maor Veitsman
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

from __future__ import absolute_import

import logging
import psutil

from s_tui.sources.source import Source


class FreqSource(Source):
    """Source class implementing CPU frequency information polling"""

    def __init__(self):
        self.is_available = True
        if not hasattr(psutil, "cpu_freq") and psutil.cpu_freq():
            self.is_available = False
            logging.debug("cpu_freq is not available from psutil")
            return

        Source.__init__(self)

        self.name = "Frequency"
        self.measurement_unit = "MHz"
        self.pallet = (
            "freq light",
            "freq dark",
            "freq light smooth",
            "freq dark smooth",
        )

        # Check if psutil.cpu_freq is available.
        # +1 for average frequency
        self.last_measurement = [0] * len(psutil.cpu_freq(True))
        if psutil.cpu_freq(False):
            self.last_measurement.append(0)

        self.top_freq = psutil.cpu_freq().max
        self.max_freq = self.top_freq

        if self.top_freq == 0.0:
            # If top freq not available, take the current as top
            if max(self.last_measurement) >= 0:
                self.max_freq = max(self.last_measurement)

        self.available_sensors = ["Avg"]
        for core_id, _ in enumerate(psutil.cpu_freq(True)):
            self.available_sensors.append("Core " + str(core_id))

    def update(self):
        self.last_measurement = [psutil.cpu_freq(False).current]
        for core in psutil.cpu_freq(True):
            self.last_measurement.append(core.current)

    def get_maximum(self):
        return self.max_freq

    def get_top(self):
        logging.debug("Returning top %s", self.top_freq)
        return self.top_freq



#!/usr/bin/env python

# Copyright (C) 2017-2020 Alex Manuskin, Maor Veitsman
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

from __future__ import absolute_import

import logging
import psutil

from s_tui.sources.source import Source


class UtilSource(Source):
    def __init__(self):
        if not hasattr(psutil, "cpu_percent") and psutil.cpu_percent():
            self.is_available = False
            logging.debug("cpu utilization is not available from psutil")
            return

        Source.__init__(self)

        self.name = "Util"
        self.measurement_unit = "%"
        self.pallet = (
            "util light",
            "util dark",
            "util light smooth",
            "util dark smooth",
        )

        self.last_measurement = [0] * (psutil.cpu_count() + 1)

        self.available_sensors = ["Avg"]
        for core_id in range(psutil.cpu_count()):
            self.available_sensors.append("Core " + str(core_id))

    def update(self):
        self.last_measurement = [psutil.cpu_percent(interval=0.0, percpu=False)]
        for util in psutil.cpu_percent(interval=0.0, percpu=True):
            logging.info("Core id util %s", util)
            self.last_measurement.append(float(util))

        logging.info("Utilization recorded %s", self.last_measurement)

    def get_is_available(self):
        return self.is_available

    def get_top(self):
        # Util can only be as high as 100%
        return 100



#!/usr/bin/env python
#
# Copyright (C) 2017-2020 Alex Manuskin, Gil Tsuker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

"""A class to control the options of stress in a menu
"""

from __future__ import print_function
import re
import logging

import psutil
import urwid


class StressMenu:
    MAX_TITLE_LEN = 50

    def __init__(self, return_fn, stress_exe):
        self.return_fn = return_fn

        self.stress_exe = stress_exe

        self.time_out = "none"
        self.sqrt_workers = "1"
        try:
            self.sqrt_workers = str(psutil.cpu_count())
            logging.info("num cpus %s", self.sqrt_workers)
        except (IOError, OSError) as err:
            logging.debug(err)

        self.sync_workers = "0"
        self.memory_workers = "0"
        self.malloc_byte = "256M"
        self.byte_touch_cnt = "4096"
        self.malloc_delay = "none"
        self.no_malloc = False
        self.write_workers = "0"
        self.write_bytes = "1G"

        self.time_out_ctrl = urwid.Edit("Time out [sec]: ", self.time_out)
        self.sqrt_workers_ctrl = urwid.Edit("Sqrt() worker count: ", self.sqrt_workers)
        self.sync_workers_ctrl = urwid.Edit("Sync() worker count: ", self.sync_workers)
        self.memory_workers_ctrl = urwid.Edit(
            "Malloc() / Free() worker count: ", self.memory_workers
        )
        self.malloc_byte_ctrl = urwid.Edit("   Bytes per malloc*: ", self.malloc_byte)
        self.byte_touch_cnt_ctrl = urwid.Edit(
            "   Touch a byte after * bytes: ", self.byte_touch_cnt
        )
        self.malloc_delay_ctrl = urwid.Edit(
            "   Sleep time between Free() [sec]: ", self.malloc_delay
        )
        self.no_malloc_ctrl = urwid.CheckBox(
            '"dirty" the memory \ninstead of free / alloc', self.no_malloc
        )
        self.write_workers_ctrl = urwid.Edit(
            "Write() / Unlink() worker count: ", self.write_workers
        )
        self.write_bytes_ctrl = urwid.Edit("   Byte per Write(): ", self.write_bytes)

        default_button = urwid.Button("Default", on_press=self.on_default)
        default_button._label.align = "center"

        save_button = urwid.Button("Save", on_press=self.on_save)
        save_button._label.align = "center"

        cancel_button = urwid.Button("Cancel", on_press=self.on_cancel)
        cancel_button._label.align = "center"

        if_buttons = urwid.Columns([default_button, save_button, cancel_button])

        title = urwid.Text(("bold text", "  Stress Options  \n"), "center")

        self.titles = [
            title,
            self.time_out_ctrl,
            urwid.Divider("-"),
            self.sqrt_workers_ctrl,
            urwid.Divider("-"),
            self.sync_workers_ctrl,
            urwid.Divider("-"),
            self.memory_workers_ctrl,
            urwid.Divider(),
            self.malloc_byte_ctrl,
            urwid.Divider(),
            self.byte_touch_cnt_ctrl,
            urwid.Divider(),
            self.malloc_delay_ctrl,
            urwid.Divider(),
            self.no_malloc_ctrl,
            urwid.Divider("-"),
            self.write_workers_ctrl,
            urwid.Divider(),
            self.write_bytes_ctrl,
            urwid.Divider("-"),
            if_buttons,
        ]

        self.main_window = urwid.LineBox(urwid.ListBox(self.titles))

    def set_edit_texts(self):
        self.time_out_ctrl.set_edit_text(self.time_out)
        self.sqrt_workers_ctrl.set_edit_text(self.sqrt_workers)
        self.sync_workers_ctrl.set_edit_text(self.sync_workers)
        self.memory_workers_ctrl.set_edit_text(self.memory_workers)
        self.malloc_byte_ctrl.set_edit_text(self.malloc_byte)
        self.byte_touch_cnt_ctrl.set_edit_text(self.byte_touch_cnt)
        self.malloc_delay_ctrl.set_edit_text(self.malloc_delay)
        self.no_malloc_ctrl.set_state(self.no_malloc)
        self.write_workers_ctrl.set_edit_text(self.write_workers)
        self.write_bytes_ctrl.set_edit_text(self.write_bytes)

    def on_default(self, _):
        self.time_out = "none"
        self.sqrt_workers = "1"
        self.sync_workers = "0"
        self.memory_workers = "0"
        self.malloc_byte = "256M"
        self.byte_touch_cnt = "4096"
        self.malloc_delay = "none"
        self.no_malloc = False
        self.write_workers = "0"
        self.write_bytes = "1G"

        self.set_edit_texts()
        self.return_fn()

    def get_size(self):
        return len(self.titles) + 5, self.MAX_TITLE_LEN

    def on_save(self, _):
        self.time_out = self.get_pos_num(self.time_out_ctrl.get_edit_text(), "none")
        self.sqrt_workers = self.get_pos_num(
            self.sqrt_workers_ctrl.get_edit_text(), "4"
        )
        self.sync_workers = self.get_pos_num(
            self.sync_workers_ctrl.get_edit_text(), "0"
        )
        self.memory_workers = self.get_pos_num(
            self.memory_workers_ctrl.get_edit_text(), "0"
        )
        self.malloc_byte = self.get_valid_byte(
            self.malloc_byte_ctrl.get_edit_text(), "256M"
        )
        self.byte_touch_cnt = self.get_valid_byte(
            self.byte_touch_cnt_ctrl.get_edit_text(), "4096"
        )
        self.malloc_delay = self.get_pos_num(
            self.malloc_delay_ctrl.get_edit_text(), "none"
        )
        self.no_malloc = self.no_malloc_ctrl.get_state()
        self.write_workers = self.get_pos_num(
            self.write_workers_ctrl.get_edit_text(), "0"
        )
        self.write_bytes = self.get_valid_byte(
            self.write_bytes_ctrl.get_edit_text(), "1G"
        )

        self.set_edit_texts()
        self.return_fn()

    def on_cancel(self, _):
        self.set_edit_texts()
        self.return_fn()

    def get_stress_cmd(self):
        stress_cmd = [self.stress_exe]
        if int(self.sqrt_workers) > 0:
            stress_cmd.append("-c")
            stress_cmd.append(self.sqrt_workers)

        if int(self.sync_workers) > 0:
            stress_cmd.append("-i")
            stress_cmd.append(self.sync_workers)

        if int(self.memory_workers) > 0:
            stress_cmd.append("--vm")
            stress_cmd.append(self.memory_workers)
            stress_cmd.append("--vm-bytes")
            stress_cmd.append(self.malloc_byte)
            stress_cmd.append("--vm-stride")
            stress_cmd.append(self.byte_touch_cnt)

        if self.no_malloc:
            stress_cmd.append("--vm-keep")

        if int(self.write_workers) > 0:
            stress_cmd.append("--hdd")
            stress_cmd.append(self.write_workers)
            stress_cmd.append("--hdd-bytes")
            stress_cmd.append(self.write_bytes)

        if self.time_out != "none":
            stress_cmd.append("-t")
            stress_cmd.append(self.time_out)

        return stress_cmd

    @staticmethod
    def get_pos_num(num, default):
        num_valid = re.match(r"\A([0-9]+)\Z", num, re.I)
        if num_valid or (num == "none" and default == "none"):
            return num
        return default

    @staticmethod
    def get_valid_byte(num, default):
        """check if the format of number is (num)(G|m|B) i.e 500GB, 200mb. 400
        etc.."""
        num_valid = re.match(r"\A([0-9]+)(M|G|m|g|)(B|b|\b)\Z", num, re.I)
        if num_valid:
            return num
        return default



#!/usr/bin/env python

# Copyright (C) 2017-2020 Alex Manuskin, Gil Tsuker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

"""ScalableBarGraph Class extends urwid.BarGraph so that
the current size of the bar graph is also obtainable
get_size() - returns the tuple (row, col)
"""

from __future__ import absolute_import

import urwid


class ScalableBarGraph(urwid.BarGraph):
    """Scale the graph according to screen size"""

    _size = (0, 0)

    def render(self, size, focus=False):
        canvas = super(ScalableBarGraph, self).render(size, focus)
        new_size = (int(canvas.rows()), int(canvas.cols()))
        old_size = self._size
        # check if to raise *on_resize* event
        if new_size != old_size:
            self.on_resize(new_size)
        self._size = new_size
        return canvas

    def calculate_bar_widths(self, size, bardata):
        """
        Return a list of bar widths, one for each bar in data.

        If self.bar_width is None this implementation will stretch
        the bars across the available space specified by maxcol.
        """
        (maxcol, _) = size

        if self.bar_width is not None:
            return [self.bar_width] * min(len(bardata), int(maxcol / self.bar_width))

        if len(bardata) >= maxcol:
            return [1] * maxcol

        widths = []
        grow = maxcol
        remain = len(bardata)
        for _ in bardata:
            w = int(float(grow) / remain + 0.5)
            widths.append(w)
            grow -= w
            remain -= 1
        return widths

    def get_size(self):
        return self._size

    def on_resize(self, new_size):
        pass  # place folder for any future implantation


class LabeledBarGraphVector(urwid.WidgetPlaceholder):
    """Add option to add labels for X and Y axes"""

    def __init__(
        self, title, sub_title_list, y_label, bar_graph_vector, visible_graph_list
    ):
        for bar_graph in bar_graph_vector:
            if not isinstance(bar_graph, ScalableBarGraph):
                raise Exception("graph vector items must be ScalableBarGraph")
        if not self.check_label(y_label):
            raise Exception("Y label must be a valid label")

        self.visible_graph_list = visible_graph_list
        self.bar_graph_vector = []
        self.set_graph(bar_graph_vector)

        self.y_label_and_graphs = urwid.WidgetPlaceholder(urwid.Columns([]))
        self.y_label = []
        self.set_y_label(y_label)

        list_w = urwid.ListBox(urwid.SimpleFocusListWalker([]))
        self.title = urwid.WidgetPlaceholder(list_w)
        self.sub_title_list = sub_title_list
        self.set_title(title)

        super(LabeledBarGraphVector, self).__init__(urwid.Pile([]))
        self.set_visible_graphs(visible_graph_list)

    def set_title(self, title):
        if not title:
            return
        title_text_w = urwid.Text(title, align="center")
        list_w = urwid.SimpleFocusListWalker([title_text_w])
        self.title.original_widget = urwid.ListBox(list_w)

    def set_y_label(self, y_label):
        if not y_label:
            text = urwid.Text("1")
            pile = urwid.Pile([urwid.ListBox([text])])
            self.y_label = ("fixed", 1, pile)
            return

        str_y_label = [str(i) for i in y_label]
        y_label_nums = str_y_label[1:]
        y_list_walker = [(1, urwid.ListBox([urwid.Text(str_y_label[0])]))]

        for num in y_label_nums:
            y_list_walker = [urwid.ListBox([urwid.Text(num)])] + y_list_walker

        y_list_walker = urwid.Pile(y_list_walker, focus_item=0)
        y_scale_len = len(max(str_y_label, key=len))

        self.y_label = ("fixed", y_scale_len, y_list_walker)

    def set_visible_graphs(self, visible_graph_list=None):
        """Show a column of the graph selected for display"""
        if visible_graph_list is None:
            visible_graph_list = self.visible_graph_list

        vline = urwid.AttrWrap(urwid.SolidFill("|"), "line")

        graph_vector_column_list = []
        for state, graph, sub_title in zip(
            visible_graph_list, self.bar_graph_vector, self.sub_title_list
        ):
            if state:
                text_w = urwid.Text(sub_title, align="center")
                sub_title_widget = urwid.ListBox([text_w])
                graph_a = [("fixed", 1, sub_title_widget), ("weight", 1, graph)]
                graph_and_title = urwid.Pile(graph_a)
                graph_vector_column_list.append(("weight", 1, graph_and_title))
                graph_vector_column_list.append(("fixed", 1, vline))

        # if all sub graph are disabled
        if not graph_vector_column_list:
            self.visible_graph_list = visible_graph_list
            self.original_widget = urwid.Pile([])
            return

        # remove the last vertical line separator
        graph_vector_column_list.pop()

        y_label_a = ("weight", 1, urwid.Columns(graph_vector_column_list))
        y_label_and_graphs = [self.y_label, y_label_a]
        column_w = urwid.Columns(y_label_and_graphs, dividechars=1)
        y_label_and_graphs_widget = urwid.WidgetPlaceholder(column_w)

        init_widget = urwid.Pile(
            [("fixed", 1, self.title), ("weight", 1, y_label_and_graphs_widget)]
        )

        self.visible_graph_list = visible_graph_list
        self.original_widget = init_widget

    def set_graph(self, graph_vector):
        self.bar_graph_vector = graph_vector

    @staticmethod
    def check_label(label):
        if len(label) >= 2 and not (None in label) or not label or label is None:
            return True

        return False



#!/usr/bin/env python

# Copyright (C) 2017-2020 Alex Manuskin, Maor Veitsman
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
""" This module implements a fan source """


from __future__ import absolute_import

import logging
import psutil
from s_tui.sources.source import Source


class FanSource(Source):
    """Source for fan information"""

    def __init__(self):
        try:
            if psutil.sensors_fans():
                self.is_available = True
        except AttributeError:
            self.is_available = False
            logging.debug("Fans sensors is not available from psutil")
            return

        Source.__init__(self)

        self.name = "Fan"
        self.measurement_unit = "RPM"
        self.pallet = ("fan light", "fan dark", "fan light smooth", "fan dark smooth")

        sensors_dict = dict()
        try:
            sensors_dict = psutil.sensors_fans()
        except IOError:
            logging.debug("Unable to create sensors dict")
            self.is_available = False
            return
        if not sensors_dict:
            self.is_available = False
            return

        for key, value in sensors_dict.items():
            sensor_name = key
            for sensor_idx, sensor in enumerate(value):
                sensor_label = sensor.label

                full_name = ""
                if not sensor_label:
                    full_name = sensor_name + "," + str(sensor_idx)
                else:
                    full_name = sensor_label

                logging.debug("Fan sensor name %s", full_name)

                self.available_sensors.append(full_name)

        self.last_measurement = [0] * len(self.available_sensors)

    def update(self):
        sample = psutil.sensors_fans()
        self.last_measurement = []
        for sensor in sample.values():
            for minor_sensor in sensor:
                # Ignore unreasonalbe fan speeds
                if minor_sensor.current > 10000:
                    continue
                self.last_measurement.append(int(minor_sensor.current))

    def get_edge_triggered(self):
        return False

    def get_top(self):
        return 1



#!/usr/bin/env python

# Copyright (C) 2017-2020 Alex Manuskin, Gil Tsuker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

import urwid
from collections import OrderedDict


class SummaryTextList:
    MAX_LABEL_L = 12

    def __init__(self, source, visible_sensors_list):
        self.source = source
        self.visible_summaries = OrderedDict()
        keys = list(self.source.get_summary().keys())
        # The title is the first in the list
        self.visible_summaries[keys[0]] = any(visible_sensors_list)
        # All others according to initial visibility
        for key, visible in zip(keys[1:], visible_sensors_list):
            self.visible_summaries[key] = visible

        # We keep a dict of all the items in the summary list
        self.summary_text_items = OrderedDict()

    def get_text_item_list(self):
        summery_text_list = []
        for key, val in self.source.get_summary().items():
            label_w = urwid.Text(str(key[0 : self.MAX_LABEL_L]))
            value_w = urwid.Text(str(val), align="right")
            # This can be accessed by the update method
            self.summary_text_items[key] = value_w
            col_w = urwid.Columns([("weight", 1.5, label_w), value_w])
            try:
                _ = self.visible_summaries[key]
            except KeyError:
                # If an unknown key appears, add it to list
                self.visible_summaries[key] = True
            if self.visible_summaries[key]:
                summery_text_list.append(col_w)

        return summery_text_list

    def update_visibility(self, visible_sensors):
        keys = list(self.visible_summaries.keys())
        self.visible_summaries[keys[0]] = any(visible_sensors)
        # Do not change visibility of title
        for sensor, visible in zip(keys[1:], visible_sensors):
            self.visible_summaries[sensor] = visible

    def update(self):
        for key, val in self.source.get_summary().items():
            if key in self.summary_text_items:
                self.summary_text_items[key].set_text(str(val))

    def get_is_available(self):
        return self.source.get_is_available()



#!/usr/bin/env python

# Copyright (C) 2017-2020 Alex Manuskin, Gil Tsuker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

from __future__ import absolute_import

from s_tui.sturwid.complex_bar_graph import LabeledBarGraphVector
from s_tui.sturwid.complex_bar_graph import ScalableBarGraph
import logging
import math

logger = logging.getLogger(__name__)


class BarGraphVector(LabeledBarGraphVector):
    @staticmethod
    def append_latest_value(values, new_val):
        values.append(new_val)
        return values[1:]

    MAX_SAMPLES = 300
    SCALE_DENSITY = 5

    def __init__(
        self,
        source,
        regular_colors,
        graph_count,
        visible_graph_list,
        alert_colors=None,
        bar_width=1,
    ):
        self.source = source
        self.graph_count = graph_count
        self.graph_name = self.source.get_source_name()
        self.measurement_unit = self.source.get_measurement_unit()

        self.num_samples = self.MAX_SAMPLES

        self.graph_data = []
        # We must create new instances for each list
        for _ in range(graph_count):
            self.graph_data.append([0] * self.num_samples)

        # Set max to 1 as default
        self.graph_max = 1

        self.color_a = regular_colors[0]
        self.color_b = regular_colors[1]
        self.smooth_a = regular_colors[2]
        self.smooth_b = regular_colors[3]

        if alert_colors:
            self.alert_colors = alert_colors
        else:
            self.alert_colors = regular_colors
        self.regular_colors = regular_colors

        self.satt = None

        y_label = []

        graph_title = self.graph_name + " [" + self.measurement_unit + "]"
        sub_title_list = self.source.get_sensor_list()

        # create several different instances of salable bar graph
        w = []
        for _ in range(graph_count):
            graph = ScalableBarGraph(["bg background", self.color_a, self.color_b])
            w.append(graph)

        super(BarGraphVector, self).__init__(
            graph_title, sub_title_list, y_label, w, visible_graph_list
        )

        for graph in self.bar_graph_vector:
            graph.set_bar_width(bar_width)

        self.color_counter_vector = [0] * graph_count

    def _set_colors(self, colors):
        self.color_a = colors[0]
        self.color_b = colors[1]
        self.smooth_a = colors[2]
        self.smooth_b = colors[3]
        if self.satt:
            self.satt = {(1, 0): self.smooth_a, (2, 0): self.smooth_b}

        for graph in self.bar_graph_vector:
            graph.set_segment_attributes(
                ["bg background", self.color_a, self.color_b], satt=self.satt
            )

    def get_graph_name(self):
        return self.graph_name

    def get_measurement_unit(self):
        return self.measurement_unit

    def get_is_available(self):
        return self.source.get_is_available()

    def get_label_scale(self, min_val, max_val, size):
        """Dynamically change the scale of the graph (y label)"""
        if size < self.SCALE_DENSITY:
            label_cnt = 1
        else:
            label_cnt = int(size / self.SCALE_DENSITY)
        try:
            if max_val >= 100:
                label = [
                    int((min_val + i * (max_val - min_val) / label_cnt))
                    for i in range(label_cnt + 1)
                ]
            else:
                label = [
                    round((min_val + i * (max_val - min_val) / label_cnt), 1)
                    for i in range(label_cnt + 1)
                ]
            return label
        except ZeroDivisionError:
            logging.debug("Side label creation divided by 0")
            return ""

    def set_smooth_colors(self, smooth):
        if smooth:
            self.satt = {(1, 0): self.smooth_a, (2, 0): self.smooth_b}
        else:
            self.satt = None

        for graph in self.bar_graph_vector:
            graph.set_segment_attributes(
                ["bg background", self.color_a, self.color_b], satt=self.satt
            )

    def update(self):
        if not self.get_is_available():
            return

        # NOTE setting edge trigger causes overhead
        try:
            if self.source.get_edge_triggered():
                self._set_colors(self.alert_colors)
            else:
                self._set_colors(self.regular_colors)
        except NotImplementedError:
            pass

        current_reading = self.source.get_reading_list()
        logging.info("Reading %s", current_reading)

        y_label_size_max = 0
        local_top_value = []

        # update visible graph data, and maximum
        for graph_idx, graph in enumerate(self.bar_graph_vector):
            try:
                _ = self.visible_graph_list[graph_idx]
            except IndexError:
                # If a new graph "Appears", append it to visible
                self.visible_graph_list.append(True)
            bars = []
            if self.visible_graph_list[graph_idx]:
                self.graph_data[graph_idx] = self.append_latest_value(
                    self.graph_data[graph_idx], current_reading[graph_idx]
                )

                # Get the graph width (dimension 1)
                num_displayed_bars = graph.get_size()[1]
                visible_id = self.MAX_SAMPLES - num_displayed_bars - 1

                visible_graph_data = self.graph_data[graph_idx][visible_id:]
                local_top_value.append(max(visible_graph_data))

        update_max = False
        if self.graph_max == 1:
            self.graph_max = self.source.get_top()
            update_max = True

        local_max = math.ceil(max(local_top_value))
        if local_max > self.graph_max:
            update_max = True
            self.graph_max = local_max

        # update the graph bars
        for graph_idx, graph in enumerate(self.bar_graph_vector):
            bars = []
            if self.visible_graph_list[graph_idx]:
                # Get the graph width (dimension 1)
                num_displayed_bars = graph.get_size()[1]
                # Iterate over all the information in the graph
                if self.color_counter_vector[graph_idx] % 2 == 0:
                    for n in range(
                        self.MAX_SAMPLES - num_displayed_bars, self.MAX_SAMPLES
                    ):
                        value = round(self.graph_data[graph_idx][n], 1)
                        # toggle between two bar types
                        if n & 1:
                            bars.append([0, value])
                        else:
                            bars.append([value, 0])
                else:
                    for n in range(
                        self.MAX_SAMPLES - num_displayed_bars, self.MAX_SAMPLES
                    ):
                        value = round(self.graph_data[graph_idx][n], 1)
                        if n & 1:
                            bars.append([value, 0])
                        else:
                            bars.append([0, value])
                self.color_counter_vector[graph_idx] += 1

                graph.set_data(bars, float(self.graph_max))
                y_label_size_max = max(y_label_size_max, graph.get_size()[0])

        self.set_y_label(
            self.get_label_scale(0, self.graph_max, float(y_label_size_max))
        )
        # Only create new graphs if the maximum has changed
        if update_max:
            self.set_visible_graphs()

    def reset(self):
        self.graph_data = []
        # Like in init, we create new instances for each list
        for _ in range(self.graph_count):
            self.graph_data.append([0] * self.num_samples)
        self.graph_max = 1



#!/usr/bin/env python

# Copyright (C) 2017-2020 Alex Manuskin, Maor Veitsman
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
""" This module reads intel power measurements"""

from __future__ import absolute_import

import logging
import glob
import os
import re
from collections import namedtuple
from multiprocessing import cpu_count
from sys import byteorder
from s_tui.helper_functions import cat


INTER_RAPL_DIR = "/sys/class/powercap/intel-rapl/"
AMD_ENERGY_DIR_GLOB = "/sys/devices/platform/amd_energy.0/hwmon/hwmon*/"
MICRO_JOULE_IN_JOULE = 1000000.0


UNIT_MSR = 0xC0010299
CORE_MSR = 0xC001029A
PACKAGE_MSR = 0xC001029B
ENERGY_UNIT_MASK = 0x1F00


RaplStats = namedtuple("rapl", ["label", "current", "max"])


class RaplReader:
    def __init__(self):
        basenames = glob.glob("/sys/class/powercap/intel-rapl:*/")
        self.basenames = sorted(set({x for x in basenames}))

    def read_power(self):
        """Read power stats and return dictionary"""

        pjoin = os.path.join
        ret = list()
        for path in self.basenames:
            name = None
            try:
                name = cat(pjoin(path, "name"), fallback=None, binary=False)
            except (IOError, OSError, ValueError) as err:
                logging.warning("ignoring %r for file %r", (err, path), RuntimeWarning)
                continue
            if name:
                try:
                    current = cat(pjoin(path, "energy_uj"))
                    max_reading = 0.0
                    ret.append(RaplStats(name, float(current), max_reading))
                except (IOError, OSError, ValueError) as err:
                    logging.warning(
                        "ignoring %r for file %r", (err, path), RuntimeWarning
                    )
        return ret

    @staticmethod
    def available():
        return os.path.exists("/sys/class/powercap/intel-rapl")


class AMDEnergyReader:
    def __init__(self):
        self.inputs = list(
            zip(
                (
                    cat(filename, binary=False)
                    for filename in sorted(
                        glob.glob(AMD_ENERGY_DIR_GLOB + "energy*_label")
                    )
                ),
                sorted(glob.glob(AMD_ENERGY_DIR_GLOB + "energy*_input")),
            )
        )

        # How many socket does the system have?
        socket_number = sum(1 for label, _ in self.inputs if "socket" in label)
        self.inputs.sort(key=lambda x: self.get_input_position(x[0], socket_number))

    @staticmethod
    def match_label(label):
        return re.search(r"E(core|socket)([0-9]+)", label)

    @staticmethod
    def get_input_position(label, socket_number):
        num = int(AMDEnergyReader.match_label(label).group(2))
        if "socket" in label:
            return num
        else:
            return num + socket_number

    def read_power(self):
        ret = []
        for label, inp in self.inputs:
            value = cat(inp)
            ret.append(RaplStats(label, float(value), 0.0))
        return ret

    @staticmethod
    def available():
        return os.path.exists("/sys/devices/platform/amd_energy.0")


class AMDRaplMsrReader:
    def __init__(self):
        self.core_msr_files = {}
        self.package_msr_files = {}
        for i in range(cpu_count()):
            curr_core_id = int(
                cat(
                    "/sys/devices/system/cpu/cpu" + str(i) + "/topology/core_id",
                    binary=False,
                )
            )
            if curr_core_id not in self.core_msr_files:
                self.core_msr_files[curr_core_id] = "/dev/cpu/" + str(i) + "/msr"

            curr_package_id = int(
                cat(
                    "/sys/devices/system/cpu/cpu"
                    + str(i)
                    + "/topology/physical_package_id",
                    binary=False,
                )
            )
            if curr_package_id not in self.package_msr_files:
                self.package_msr_files[curr_package_id] = "/dev/cpu/" + str(i) + "/msr"

    @staticmethod
    def read_msr(filename, register):
        f = open(filename, "rb")
        f.seek(register)
        res = int.from_bytes(f.read(8), byteorder)
        f.close()
        return res

    def read_power(self):
        ret = []
        for i, filename in self.package_msr_files.items():
            unit_msr = self.read_msr(filename, UNIT_MSR)
            energy_factor = 0.5 ** ((unit_msr & ENERGY_UNIT_MASK) >> 8)
            package_msr = self.read_msr(filename, PACKAGE_MSR)
            ret.append(
                RaplStats(
                    "Package " + str(i + 1),
                    package_msr * energy_factor * MICRO_JOULE_IN_JOULE,
                    0.0,
                )
            )

        for i, filename in self.core_msr_files.items():
            unit_msr = self.read_msr(filename, UNIT_MSR)
            energy_factor = 0.5 ** ((unit_msr & ENERGY_UNIT_MASK) >> 8)
            core_msr = self.read_msr(filename, CORE_MSR)
            ret.append(
                RaplStats(
                    "Core " + str(i + 1),
                    core_msr * energy_factor * MICRO_JOULE_IN_JOULE,
                    0.0,
                )
            )

        return ret

    @staticmethod
    def available():
        try:
            cpuinfo = cat("/proc/cpuinfo", binary=False)
            # The reader only supports family 17h CPUs
            m = re.search(r"vendor_id[\s]+: ([A-Za-z]+)", cpuinfo)

            if not m or m is None:
                return False

            if m.group(1) != "AuthenticAMD":
                return False

            m = re.search(r"cpu family[\s]+: ([0-9]+)", cpuinfo)
            if int(m[1]) != 0x17:
                return False
        except (FileNotFoundError, PermissionError):
            return False

        # with open("/proc/cpuinfo", "rb") as cpuinfo:
        #     all_info = cpuinfo.readlines()
        #     for line in all_info:
        #         if b"vendor_id" in line:
        #             print("Vendor id", line)
        #             if b"AuthenticAMD" not in line:
        #                 return False

        #     for line in all_info:
        #         if b"cpu family" in line:
        #             print("cpu family", line)
        #             m = re.search("cpu family[\s]+: ([0-9]+)", cpuinfo)
        #             if int(m[1]) != 0x17:
        #                 return False

        # Check whether MSRs are available and we have permission to read them
        try:
            open("/dev/cpu/0/msr")
            return True
        except (FileNotFoundError, PermissionError):
            return False


def get_power_reader():
    for ReaderType in (RaplReader, AMDEnergyReader, AMDRaplMsrReader):
        if ReaderType.available():
            return ReaderType()
    return None



#!/usr/bin/env python
#
# Copyright (C) 2017-2020 Alex Manuskin, Gil Tsuker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

import os
from s_tui.sources.hook_script import ScriptHook


class ScriptHookLoader:
    """
    Loads shell scripts from a directory into ScriptHooks for a given Source
    """

    def __init__(self, dir_path):
        self.scripts_dir_path = os.path.join(dir_path, "hooks.d")

    def load_script(self, source_name, timeoutMilliseconds=0):
        """
        Return ScriptHook for source_name Source and with a ready timeout
        of timeoutMilliseconds
        """

        script_path = os.path.join(
            self.scripts_dir_path, self._source_to_script_name(source_name)
        )

        if os.path.isfile(script_path):
            return ScriptHook(script_path, timeoutMilliseconds)
        return None

    def _source_to_script_name(self, source_name):
        return source_name.lower() + ".sh"



#!/usr/bin/env python
#
# Copyright (C) 2017-2020 Alex Manuskin, Gil Tsuker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

"""A class display the help message menu
"""

from __future__ import print_function
from __future__ import absolute_import
import urwid
from s_tui.sturwid.ui_elements import ViListBox

HELP_MESSAGE = """
TUI interface:

The side bar houses the controls for the displayed graphs.\n\
At the bottom, all sensors reading are presented in text form.\n\

* Use the arrow keys or 'hjkl' to navigate the side bar
* Toggle between stressed and regular operation using the radio buttons in \
'Modes'.\n\
* If you wish to alternate stress defaults, you can do it in <Stress \
options>\n\
* Select graphs to display in the <Graphs> menu \n\
* Select summaries to display in the <Summaries> menu \n\
* Change time between updates using the 'Refresh' field\n\
* Use the <Reset> button to reset graphs and statistics\n\
* If your system supports it, you can use the UTF-8 button to get a smoother \
graph\n\
* Save your current configuration with the <Save Settings> button\n\
* Press 'q' or the <Quit> button to quit\n\
\n\
* Run `s-tui --help` to get this message and additional cli options\n\
\n\
"""

MESSAGE_LEN = 30


class HelpMenu:
    """HelpMenu is a widget containing instructions on usage of s-tui"""

    MAX_TITLE_LEN = 90

    def __init__(self, return_fn):
        self.return_fn = return_fn

        self.help_message = HELP_MESSAGE

        self.time_out_ctrl = urwid.Text(self.help_message)

        cancel_button = urwid.Button(("Exit"), on_press=self.on_cancel)
        cancel_button._label.align = "center"

        if_buttons = urwid.Columns([cancel_button])

        title = urwid.Text(("bold text", "  Help Menu  \n"), "center")

        self.titles = [title, self.time_out_ctrl, if_buttons]

        self.main_window = urwid.LineBox(ViListBox(self.titles))

    def get_size(self):
        """returns size of HelpMenu"""
        return MESSAGE_LEN + 3, self.MAX_TITLE_LEN

    def on_cancel(self, w):
        """Returns to original widget"""
        self.return_fn()



#!/usr/bin/env python
#
# Copyright (C) 2017-2020 Alex Manuskin, Gil Tsuker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

"""
A class displaying all available sensors
"""

from __future__ import print_function
from __future__ import absolute_import
import copy

import urwid
from s_tui.sturwid.ui_elements import ViListBox


class SensorsMenu:
    MAX_TITLE_LEN = 120

    def on_mode_button(self, button, state):
        pass

    def __init__(self, return_fn, source_list, default_source_conf):
        self.return_fn = return_fn

        # create the cancel and apply buttons, and put the in an urwid column
        cancel_button = urwid.Button("Cancel", on_press=self.on_cancel)
        cancel_button._label.align = "center"
        apply_button = urwid.Button("Apply", on_press=self.on_apply)
        apply_button._label.align = "center"

        if_buttons = urwid.Columns([apply_button, cancel_button])

        self.sensor_status_dict = {}
        sensor_column_list = []
        self.sensor_button_dict = {}
        self.active_sensors = {}
        for source in source_list:
            source_name = source.get_source_name()

            # get the saves sensor visibility list
            if default_source_conf[source_name]:
                # print(str(default_source_conf[source_name]))
                self.sensor_status_dict[source_name] = copy.deepcopy(
                    default_source_conf[source_name]
                )
            else:
                self.sensor_status_dict[source_name] = [True] * len(
                    source.get_sensor_list()
                )

            self.sensor_button_dict[source_name] = []
            self.active_sensors[source_name] = []

            # add the title at the head of the checkbox column
            sensor_title_str = source_name
            sensor_title = urwid.Text(("bold text", sensor_title_str), "center")

            # create the checkbox buttons with the saved visibility
            for sensor, s_tatus in zip(
                source.get_sensor_list(), self.sensor_status_dict[source_name]
            ):
                cb = urwid.CheckBox(sensor, s_tatus)
                self.sensor_button_dict[source_name].append(cb)
                self.active_sensors[source_name].append(s_tatus)

            sensor_title_and_buttons = [sensor_title] + self.sensor_button_dict[
                source_name
            ]
            listw = urwid.SimpleFocusListWalker(sensor_title_and_buttons)

            sensor_column_list.append(urwid.Pile(listw))

        sensor_select_widget = urwid.Columns(sensor_column_list)

        list_temp = [sensor_select_widget, if_buttons]
        listw = urwid.SimpleFocusListWalker(list_temp)
        self.main_window = urwid.LineBox(ViListBox(listw))

        max_height = 6
        for sensor, s_tatus in self.active_sensors.items():
            max_height = max(max_height, len(s_tatus) + 6)

        self.size = max_height, self.MAX_TITLE_LEN

    def get_size(self):
        return self.size

    def set_checkbox_value(self):
        for sensor, sensor_cb in self.sensor_button_dict.items():
            sensor_cb_next_state = self.active_sensors[sensor]
            for checkbox, state in zip(sensor_cb, sensor_cb_next_state):
                checkbox.set_state(state)

    def on_cancel(self, w):
        self.set_checkbox_value()
        self.return_fn(update=False)

    def on_apply(self, w):
        update_sensor_visibility = False
        for s_name, sensor_buttons in self.sensor_button_dict.items():
            cb_sensor_visibility = []
            for sensor_cb in sensor_buttons:
                cb_sensor_visibility.append(sensor_cb.get_state())

                changed_state = cb_sensor_visibility != self.active_sensors[s_name]
                update_sensor_visibility |= changed_state

            self.active_sensors[s_name] = cb_sensor_visibility

        self.set_checkbox_value()
        self.return_fn(update=update_sensor_visibility)
